/**
 * @file
 * VuoList_VuoGradientNoise C type definition.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoGradientNoise_H
#define VuoList_VuoGradientNoise_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoGradientNoise VuoList_VuoGradientNoise
 * A list of @ref VuoGradientNoise elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoGradientNoise_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoGradientNoise elements.
 */
typedef const struct VuoList_VuoGradientNoise_struct { void *l; } * VuoList_VuoGradientNoise;
/// @}
#endif

/**
 * Creates a new list of @ref VuoGradientNoise elements.
 */
VuoList_VuoGradientNoise VuoListCreate_VuoGradientNoise(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoGradientNoise VuoListCopy_VuoGradientNoise(const VuoList_VuoGradientNoise list);

/**
 * Returns the @ref VuoGradientNoise at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoGradientNoise VuoListGetValue_VuoGradientNoise(const VuoList_VuoGradientNoise list, const unsigned long index);

/**
 * Changes the @ref VuoGradientNoise at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoGradientNoise(const VuoList_VuoGradientNoise list, const VuoGradientNoise value, const unsigned long index);

/**
 * Inserts the `VuoGradientNoise` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoGradientNoise(const VuoList_VuoGradientNoise list, const VuoGradientNoise value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoGradientNoise(VuoList_VuoGradientNoise list, const VuoGradientNoise value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoGradientNoise(VuoList_VuoGradientNoise list, const VuoGradientNoise value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoGradientNoise(VuoList_VuoGradientNoise list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoGradientNoise(VuoList_VuoGradientNoise list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoGradientNoise(VuoList_VuoGradientNoise list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoGradientNoise(VuoList_VuoGradientNoise list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoGradientNoise(VuoList_VuoGradientNoise list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoGradientNoise(VuoList_VuoGradientNoise list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoGradientNoise(VuoList_VuoGradientNoise list);

/**
 * Removes the `VuoGradientNoise` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoGradientNoise(VuoList_VuoGradientNoise list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoGradientNoise(const VuoList_VuoGradientNoise list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoGradientNoise VuoList_VuoGradientNoise_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoGradientNoise_getJson(const VuoList_VuoGradientNoise value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoGradientNoise_getSummary(const VuoList_VuoGradientNoise value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoGradientNoise VuoList_VuoGradientNoise_makeFromString(const char *str);
char * VuoList_VuoGradientNoise_getString(const VuoList_VuoGradientNoise value);
/// @}

/**
 * @}
 */

#endif
