/**
 * @file
 * VuoList_VuoReal C type definition.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoReal_H
#define VuoList_VuoReal_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoReal VuoList_VuoReal
 * A list of @ref VuoReal elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoReal_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoReal elements.
 */
typedef const struct VuoList_VuoReal_struct { void *l; } * VuoList_VuoReal;
/// @}
#endif

/**
 * Creates a new list of @ref VuoReal elements.
 */
VuoList_VuoReal VuoListCreate_VuoReal(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoReal VuoListCopy_VuoReal(const VuoList_VuoReal list);

/**
 * Returns the @ref VuoReal at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoReal VuoListGetValue_VuoReal(const VuoList_VuoReal list, const unsigned long index);

/**
 * Changes the @ref VuoReal at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoReal(const VuoList_VuoReal list, const VuoReal value, const unsigned long index);

/**
 * Inserts the `VuoReal` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoReal(const VuoList_VuoReal list, const VuoReal value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoReal(VuoList_VuoReal list, const VuoReal value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoReal(VuoList_VuoReal list, const VuoReal value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoReal(VuoList_VuoReal list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoReal(VuoList_VuoReal list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoReal(VuoList_VuoReal list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoReal(VuoList_VuoReal list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoReal(VuoList_VuoReal list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoReal(VuoList_VuoReal list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoReal(VuoList_VuoReal list);

/**
 * Removes the `VuoReal` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoReal(VuoList_VuoReal list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoReal(const VuoList_VuoReal list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoReal VuoList_VuoReal_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoReal_getJson(const VuoList_VuoReal value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoReal_getSummary(const VuoList_VuoReal value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoReal VuoList_VuoReal_makeFromString(const char *str);
char * VuoList_VuoReal_getString(const VuoList_VuoReal value);
/// @}

/**
 * @}
 */

#endif
