/**
 * @file
 * VuoList_VuoTransform2d C type definition.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoTransform2d_H
#define VuoList_VuoTransform2d_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoTransform2d VuoList_VuoTransform2d
 * A list of @ref VuoTransform2d elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoTransform2d_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoTransform2d elements.
 */
typedef const struct VuoList_VuoTransform2d_struct { void *l; } * VuoList_VuoTransform2d;
/// @}
#endif

/**
 * Creates a new list of @ref VuoTransform2d elements.
 */
VuoList_VuoTransform2d VuoListCreate_VuoTransform2d(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoTransform2d VuoListCopy_VuoTransform2d(const VuoList_VuoTransform2d list);

/**
 * Returns the @ref VuoTransform2d at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoTransform2d VuoListGetValue_VuoTransform2d(const VuoList_VuoTransform2d list, const unsigned long index);

/**
 * Changes the @ref VuoTransform2d at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoTransform2d(const VuoList_VuoTransform2d list, const VuoTransform2d value, const unsigned long index);

/**
 * Inserts the `VuoTransform2d` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoTransform2d(const VuoList_VuoTransform2d list, const VuoTransform2d value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoTransform2d(VuoList_VuoTransform2d list, const VuoTransform2d value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoTransform2d(VuoList_VuoTransform2d list, const VuoTransform2d value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoTransform2d(VuoList_VuoTransform2d list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoTransform2d(VuoList_VuoTransform2d list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoTransform2d(VuoList_VuoTransform2d list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoTransform2d(VuoList_VuoTransform2d list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoTransform2d(VuoList_VuoTransform2d list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoTransform2d(VuoList_VuoTransform2d list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoTransform2d(VuoList_VuoTransform2d list);

/**
 * Removes the `VuoTransform2d` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoTransform2d(VuoList_VuoTransform2d list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoTransform2d(const VuoList_VuoTransform2d list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoTransform2d VuoList_VuoTransform2d_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoTransform2d_getJson(const VuoList_VuoTransform2d value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoTransform2d_getSummary(const VuoList_VuoTransform2d value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoTransform2d VuoList_VuoTransform2d_makeFromString(const char *str);
char * VuoList_VuoTransform2d_getString(const VuoList_VuoTransform2d value);
/// @}

/**
 * @}
 */

#endif
